<?php   
if (!defined('_PS_VERSION_'))
	exit;

class NovitaxStock extends Module
{
	public function __construct()
	{
		$this->name = 'novitaxstock';
		$this->tab = 'others';
		$this->version = '1.0';
		$this->author = 'Novitax Kft.';
		$this->need_instance = 1;
		$this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_); 
		$this->bootstrap = true;

		parent::__construct();

		$this->displayName = $this->l('Wintax/RAXLA - Prestashop integráció');
		$this->description = $this->l('Novitax RAXLA által kezelt készletinformációk szinkronizációja Prestashop felé, valamint számlázandó megrendelések átadása RAXLA felé.');

		$this->confirmUninstall = $this->l('Biztosan szeretnéd eltávolítani a modult?');

		$key = Configuration::get('NOVITAX_KEY');
		if (!isset($key) || empty($key))
		  $this->warning = $this->l('A modul használatának megkezdéséhez be kell állítanod egy kulcsot.');    
	}


	public function install()
	{

		if (parent::install()) {
			return (
			  $this->installDb() &&
			  $this->registerHook('actionOrderStatusUpdate') &&
			  $this->registerHook('displayAdminOrder') &&
			  $this->registerHook('actionDispatcher')
			);
		}

		return false;
	}


	public function uninstall()
	{
		Configuration::deleteByName('NOVITAX_KEY');

		return    $this->uninstallDb() &&
		    parent::uninstall();
	}


	public function installDb()
	{
		return Db::getInstance()->execute(
		    'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.bqSQL($this->name).' (
		    `id_order` INTEGER(10) NOT NULL AUTO_INCREMENT,
		    `status` VARCHAR(10) DEFAULT NULL,
		    `xml` TEXT DEFAULT NULL,
		    `invoice_num` VARCHAR(100) DEFAULT NULL,
		    PRIMARY KEY(`id_order`))
		    ENGINE='._MYSQL_ENGINE_.' default CHARSET=utf8'
		);
	}


	public function uninstallDb()
	{
		return Db::getInstance()->execute('DROP TABLE IF EXISTS '._DB_PREFIX_.bqSQL($this->name));
	}


	public function getContent()
	{
		$output = null;

		if (Tools::isSubmit('submit'.$this->name))
		{
			$key = strval(Tools::getValue('NOVITAX_KEY'));
			if (!$key
			|| empty($key)
			|| !Validate::isGenericName($key))
				$output .= $this->displayError($this->l('Érvénytelen kulcs'));
			else
			{
				Configuration::updateValue('NOVITAX_KEY', $key);
				$output .= $this->displayConfirmation($this->l('A beállítások frissítése megtörtént.'));
			}
		}
		$this->context->controller->addJS(_MODULE_DIR_.$this->name.'/views/js/novitaxstock.js');
		return $output.$this->displayForm();
	}


	public function displayForm()
	{
		// Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

		// Init Fields form array
		$fields_form[0]['form'] = array(
		  'legend' => array(
		      'title' => $this->l('Beállítások')
		  ),
		  'input' => array(
		      array(
		          'type' => 'html',
		          'name' => $this->l('Írj be, vagy generálj egy véletlenszerű API kulcsot - ezt a kulcsot kell beállítani majd a RAXLA-ban.')
		      ),
		      array(
		          'type' => 'text',
		          'label' => $this->l('Novitax API kulcs'),
		          'name' => 'NOVITAX_KEY',
		          'size' => 32,
		          'required' => true,
		          'desc' => $this->l('Vigyázz erre a kulcsra!')
		      ),
		      array(
		          'type' => 'html',
		          'name' => 'generate',
		          'html_content' => '<input type="button" value="Generálás" id="novitax_generate_key" />'
		      ),
		  ),
		  'submit' => array(
		      'title' => $this->l('Save'),
		      'class' => 'btn btn-default pull-right'
		  )
		);

		$helper = new HelperForm();

		// Module, token and currentIndex
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

		// Language
		$helper->default_form_language = $default_lang;
		$helper->allow_employee_form_lang = $default_lang;

		// Title and toolbar
		$helper->title = $this->displayName;
		$helper->show_toolbar = true;        // false -> remove toolbar
		$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper->submit_action = 'submit'.$this->name;
		$helper->toolbar_btn = array(
		  'save' =>
		  array(
		      'desc' => $this->l('Save'),
		      'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
		      '&token='.Tools::getAdminTokenLite('AdminModules'),
		  ),
		  'back' => array(
		      'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
		      'desc' => $this->l('Back to list')
		  )
		);

		// Load current value
		$helper->fields_value['NOVITAX_KEY'] = Configuration::get('NOVITAX_KEY');

		return $helper->generateForm($fields_form);
	}  


	public function hookActionOrderStatusUpdate($params)
	{
		$order_id = (int)$params['id_order'];
		$status = $params['newOrderStatus']->name;

		// $params['newOrderStatus']->id == Configuration::get('PS_OS_WS_PAYMENT')

		if ($status == 'Delivered') {

			$invoice_state = '';
			$results = Db::getInstance()->getRow('SELECT status FROM '._DB_PREFIX_.bqSQL($this->name) . ' WHERE id_order = ' . $order_id);
			if ($results && !empty($results['status'])) $invoice_state = $results['status'];

			if (in_array($invoice_state, array('new', ''))) {
				$xml = $this->generate_invoice($order_id);
				Db::getInstance()->execute('REPLACE INTO '._DB_PREFIX_.bqSQL($this->name) . ' SET status = "new", xml = "' . pSQL($xml, true) . '", id_order = ' . $order_id);
			}


		}
	    
	}


	public function hookDisplayAdminOrder($params) {

		$order_id = $params['id_order'];

		$order = Db::getInstance()->getRow('SELECT status, invoice_num FROM '._DB_PREFIX_.bqSQL($this->name) . ' WHERE id_order = ' . $order_id);

		$s = '';
		$s .= '<div class="row"><div class="col-lg-12"><div class="panel"><div class="panel-heading"><i class="icon-info"></i> Wintax Integráció</div>';
		$s .= '<dl class="well list-detail">';
		$s .= '<dt>Számlázási adatok státusza</dt>';
		$s .= '<dd>';

		if (!isset($order['status'])) $s .= '-';
		elseif ($order['status'] == 'new') $s .= 'számlázási adatok előkészítve';
		elseif ($order['status'] == 'sent') $s .= 'számlázási adatok elküldve Wintax felé';
		elseif ($order['status'] == 'generated') $s .= 'számla elkészült';
		else $s .= '-';

		$s .= '</dd>';

		if (!empty($order['invoice_num'])) {
			$s .= '<dt>Legenerált számla száma</dt>';
			$s .= '<dd>' . $order['invoice_num'] . '</dd>';
		}

		$s .= '</dl>';
		$s .= '</div></div></div>';

		echo $s;

	}


	//Generate XML 
	public function generate_invoice($orderId) {


		$order = new Order($orderId);

		//Build Xml

		$o = new SimpleXMLElement('<Order></Order>');

		$o->addChild('Key', $orderId);

		//Customer details
		$vevo = $o->addChild('Customer');
		$vevo->addChild('Id',$order->id_customer);

		$addresses = $vevo->addChild('Addresses');

		$invoice = $addresses->addChild('Invoice');
		$invoice_address = new Address($order->id_address_invoice);

		$invoice->addChild('Name', ($invoice_address->company) ? htmlspecialchars($invoice_address->company, ENT_XML1, 'UTF-8') : ($invoice_address->lastname .' ' . $invoice_address->firstname));
		$invoice->addChild('ZIP', $invoice_address->postcode);
		$invoice->addChild('City', $invoice_address->city);
		$invoice->addChild('Street', $invoice_address->address1);
		$invoice->addChild('StreetName', $invoice_address->address1);

		$invoice->addChild('County','');
		$invoice->addChild('Country', $invoice_address->country);

		$c = new Country();    
		$invoice->addChild('CountryCode', $c->getIsoById($invoice_address->id_country));
		$invoice->addChild('TaxNumber', $invoice_address->vat_number);



		$shipping = $addresses->addChild('Shipping');
		$shipping_address = new Address($order->id_address_delivery);

		$shipping->addChild('Name', ($shipping_address->company) ? htmlspecialchars($shipping_address->company, ENT_XML1, 'UTF-8') : ($shipping_address->lastname .' ' . $shipping_address->firstname));
		$shipping->addChild('ZIP', $shipping_address->postcode);
		$shipping->addChild('City', $shipping_address->city);
		$shipping->addChild('Street', $shipping_address->address1);
		$shipping->addChild('StreetName', $shipping_address->address1);

		$shipping->addChild('County', '');
		$shipping->addChild('Country', $shipping_address->country);

		$shipping->addChild('CountryCode', $c->getIsoById($shipping_address->id_country));


		$cur = new Currency($order->id_currency);    
		$o->addChild('Currency', $cur->iso_code);

		$o->addChild('Payment', $this->get_payment_type($order->payment));



		$ProductDetailObject = new OrderDetail;
		$order_items = $ProductDetailObject->getList($orderId);

		$items = $o->addChild('Items');  

		foreach( $order_items as $item ) {

			$i = $items->addChild('Item');
			$i->addChild('Id', $item['product_id']);
			$i->addChild('Sku', $item['product_reference']);
			$i->addChild('Name', htmlspecialchars($item['product_name']));

			$i->addChild('Unit','db');        
			$i->addChild('Quantity', $item['product_quantity']);

			$i->addChild('PriceGross', $item['unit_price_tax_incl']);

			$taxCalculator = $ProductDetailObject->getTaxCalculatorStatic($item['id_order_detail']);
			$i->addChild('Vat', $taxCalculator->getTotalRate() . '%');

		}  



		// Shipping
		if($order->total_shipping_tax_incl > 0) {

			$i = $items->addChild('Item');
			$i->addChild('Id', 'shipping-cost');
			$i->addChild('Sku', 'shipping-cost');

			$carrier = new Carrier($order->id_carrier);
			$i->addChild('Name', 'Szállítási költség - ' . htmlspecialchars($carrier->name));

			$i->addChild('Unit','db');        
			$i->addChild('Quantity',1);

			$i->addChild('PriceGross', $order->total_shipping_tax_incl);
			$i->addChild('Vat', $order->carrier_tax_rate . '%');


		}



		$xml = $o->asXML();
		$xml = substr($xml, strpos($xml, '?>') + 2);

		return $xml;

	}


	// fizetés módja
	public function get_payment_type($type) {

		switch ($type) {
		  case 'Bank wire':  // átutalás
		    return 'AT';
		    break;
		  case 'Cash on delivery (COD)':   // utánvétel
		    return 'UV';
		    break;      
		  default:
		    return 'other - ' . $type;
		    break;
		}

	}


	// kulcs összevetése a tárolttal
	public function check_key($key) {
		$novitax_key = Configuration::get('NOVITAX_KEY');
		if (!empty($key) && ($novitax_key == $key)) {
			return true;
		}
		else {
			return false;
		}
	}


	// kapcsolat ellenőrzés
	public function test_connection() {
		echo 'OK';
	}


	// számlázandó rendelések xml-jeinek összefűzése és elküldése
	public function get_invoice_data() {

		$xml = '<?xml version="1.0" encoding="UTF-8"?><Orders>';
		$ids = array();

		$results = Db::getInstance()->executeS('SELECT id_order, xml FROM '._DB_PREFIX_.bqSQL($this->name) . ' WHERE status = "new"', true, 0);

		if ($results) {
			foreach ($results as $result) {
				$xml .= $result['xml'];
				$ids[] = $result['id_order'];
			}

			if (!empty($ids)) {
				$r2 = Db::getInstance()->execute('UPDATE '._DB_PREFIX_.bqSQL($this->name) . ' SET status = "sent" WHERE id_order IN (' . implode(',', $ids). ')');      
			}

		}


		  $xml .= '</Orders>';

		  header("Content-type: text/xml; charset=utf-8");
		  echo $xml;  

	}


	// számlázott rendelések számlaszámának (pdf számláinak) fogadása
	public function send_invoice() {

		$order_id = (int)$_POST['order_id'];
		$invoice_num = $_POST['invoice_num'];

		//$file = $_POST['invoice_pdf'];

		$id_order = Db::getInstance()->getValue("SELECT id_order FROM "._DB_PREFIX_.bqSQL('orders') . " WHERE id_order = $order_id");

		if ($id_order === false) {
			header('HTTP/1.1 422 Unprocessable Entity');
			echo 'order_id not exists: ' . $order_id . "\n";
			exit;
		}


		$result = Db::getInstance()->execute('UPDATE '._DB_PREFIX_.bqSQL($this->name) . ' SET status = "generated", invoice_num = "' . pSQL($invoice_num) . '" WHERE id_order = ' . $order_id . ' LIMIT 1');      

		echo 'OK';

		
		
	}


	public function hookActionDispatcher($params) {
		$this->wintax_process();
	}


	public function wintax_process() {

		if (isset($_POST['wintax_method'])) {

			$r = $this->check_key($_POST['wintax_key']);
			if (!$r) {
				header('HTTP/1.1 401 Unauthorized');
				exit;
			}

			switch ($_POST['wintax_method']) {
			case 'update_stock':
			  $this->update_stock();
			  exit;
			case 'get_invoice_data':
			  $this->get_invoice_data();
			  exit;
			case 'send_invoice':
			  $this->send_invoice();
			  exit;                     
			case 'test_connection':
			  $this->test_connection();
			  exit;         
			default:
			  header('HTTP/1.1 400 Bad Request');
			  exit;
			}
		  
		}
		else {
			return;
		}


	}


	// készlet adatok frissítése wintax felől
	public function update_stock() {

		libxml_use_internal_errors(true);
		$xml = simplexml_load_string(stripslashes($_POST['wintax_xml']));

		if ($xml === false) {
			header('HTTP/1.1 422 Unprocessable Entity');
			echo 'malformed xml';
			exit;
		}

		$found = 0;

		foreach ($xml->StockItem as $product) {

			$sku = $product->EAN;
			$price = (int)$product->Price;
			$q = (int)$product->Amount;

			if (!empty($sku)) {

				$r = Db::getInstance()->getRow("SELECT id_product, id_product_attribute FROM " . _DB_PREFIX_ . "product_attribute WHERE reference = '" . pSQL($sku) . "'");

				if (isset($r['id_product_attribute'])) {
					StockAvailable::setQuantity($r['id_product'], $r['id_product_attribute'], $q);
					$found++;
				}
				else {
					$id_product = (int)Db::getInstance()->getValue("SELECT id_product FROM "._DB_PREFIX_."product WHERE reference = '" . pSQL($sku) . "'");      
					if ($id_product) {
						StockAvailable::setQuantity($id_product, 0, $q);
						$found++;
					}
				}

			}
		  
		}

		echo 'OK: ' . $found;
	}


}